% (c) Wojciech Paszke, ISSI
% e-mail: w.paszke@issi.uz.zgora.pl

close all  % zamykamy wszytkie okna
clear all  % czycimy wszystkie zmienne
clc % czycimy ekran
%% parametry wahada ktre jest w sali 406

 M = 4.2774;  % masa wzka [kg]
 m = 0.3211;  % masa wahadla [kg]
 l = 0.3533;  % dlugosc wahadla [m]
 Jt = 0.072;  % Jt=J+m*l^2 [kgm^2], gdzie J - moment bezwladnosci w odniesieniu do srodka ciezkosci
 gamma=10.0;  % Staa tarcia prdkosciowego [kg/s]
 c=0.023;     % staa tarcia wahada [kgm^2/s]
 g=9.81;      % przyspieszenie ziemskie [m/s^2]
 
 Mt=m+M;
 mu=Mt*Jt-m^2*l^2;
 
 % Model w przestrzeni stanw na podstawie ksiaki
 
  A=[0, 0, 1, 0;...
    0, 0, 0, 1;... 
    0, m^2*l^2*g/mu, -c*Jt/mu, -gamma*l*m/mu;...
    0, Mt*m*g*l/mu, -c*l*m/mu, -gamma*Mt/mu];

 B=[0; 0; Jt/mu; l*m/mu];

 C=[1,0,0,0; 0,1,0,0];   % dwa wyjcia: polozenie wozka i kat odchylenia wahadla
 
 D=[0;0];
 
pendulumOL=ss(A,B,C,D); % tutaj mamy model w przestrzeni stanw w jednaj strukturze

T=0:0.05:10;
U=0.2*ones(size(T)); % bedziemy symulowa przesunicie wzka o 20 czm czyli 
                     % 0.2 metra

%% Symulacja wahadla w otwartej petli

[Y,T,X]=lsim(pendulumOL,U,T);
plot(T,Y)
title('Symulacja wahada odwconego w ukadzie otwartym');
legend('kat odchylenia [rad]','polozenie wozka [m]');
grid on
% ukad wahada powinien byc niestabilny,
% mozemy sprawdzic wartosci wlasne macierzy A czyli eig(A)
% po drugie paptrzac na odpowiedz skokowa od razu mozemy zauwac e wahadlo
% jest niestabilne
% To sie zgadza bo wiekszosci przypadkow jak postawie wahadlo i nie bedzie
% sterowania to ono natychmiast opadnie

% stabilizacja: przy zalozieniu ze mamy dostep do stanu

K=place(A,B,[-1;-144.6;-10;-0.005]); % szukamy sterownika od stanu
                                     % stabilizujemy tylko te bieguny
                                     % ktre s w prawej ppaszczynie
                                     % czyli niestabilne bieguny
pendulumCL=ss(A-B*K,B,C,D);  % ukad zamkniety
T=0:0.05:100;
U=0.2*ones(size(T));

% symulujemy ukad zamkniety
[Y,T,X]=lsim(pendulumCL,U,T);
figure
plot(T,Y)
title('Symulacja wahada odwconego w ukadzie zamknitym');
legend('kat odchylenia [rad]','polozenie wozka [m]');
grid on
